﻿' •—————————————————————————————————————————————————————————————————————————————————————————————————————•
'                              National Institute of Standards and Technology
'                                          Biometric Clients Lab
' •—————————————————————————————————————————————————————————————————————————————————————————————————————•
'  File author(s):
'       Ross J. Micheals (ross.micheals@nist.gov)
'
' •—————————————————————————————————————————————————————————————————————————————————————————————————————•
' | NOTICE & DISCLAIMER                                                                                 |
' |                                                                                                     |
' | The research software provided on this web site (“software”) is provided by NIST as a public 		|
' | service. You may use, copy and distribute copies of the software in any medium, provided that you 	|
' | keep intact this entire notice. You may improve, modify and create derivative works of the software	|
' | or any portion of the software, and you may copy and distribute such modifications or works.  		|
' | Modified works should carry a notice stating that you changed the software and should note the date	|
' | and nature of any such change.  Please explicitly acknowledge the National Institute of Standards	|
' | and Technology as the source of the software.														|
' | 																									|
' | The software is expressly provided “AS IS.”  NIST MAKES NO WARRANTY OF ANY KIND, EXPRESS, IMPLIED, 	|
' | IN FACT OR ARISING BY OPERATION OF LAW, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTY OF 		|
' | MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, NON-INFRINGEMENT AND DATA ACCURACY.  NIST 		|
' | NEITHER REPRESENTS NOR WARRANTS THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR 		|
' | ERROR-FREE, OR THAT ANY DEFECTS WILL BE CORRECTED.  NIST DOES NOT WARRANT OR MAKE ANY 				|
' | REPRESENTATIONS REGARDING THE USE OF THE SOFTWARE OR THE RESULTS THEREOF, INCLUDING BUT NOT LIMITED	|
' | TO THE CORRECTNESS, ACCURACY, RELIABILITY, OR USEFULNESS OF THE SOFTWARE.							|
' | 																									|
' | You are solely responsible for determining the appropriateness of using and distributing the 		|
' | software and you assume all risks associated with its use, including but not limited to the risks	|
' | and costs of program errors, compliance with applicable laws, damage to or loss of data, programs	|
' | or equipment, and the unavailability or interruption of operation.  This software is not intended	|
' | to be used in any situation where a failure could cause risk of injury or damage to property.  The	|
' | software was developed by NIST employees.  NIST employee contributions are not subject to copyright	|
' | protection within the United States.  																|
' | 																									|
' | Specific hardware and software products identified in this open source project were used in order   |
' | to perform technology transfer and collaboration. In no case does such identification imply         |
' | recommendation or endorsement by the National Institute of Standards and Technology, nor            |
' | does it imply that the products and equipment identified are necessarily the best available for the |
' | purpose.                                                                                            |
' •—————————————————————————————————————————————————————————————————————————————————————————————————————•

Option Strict On
Option Infer On

Imports System.Runtime.Serialization
Imports System.ServiceModel
Imports System.ServiceModel.Web
Imports System.Threading
Imports System.Threading.Tasks

Imports Nist.Bcl.Wsbd


<TestClass()>
Public Class BaselineSensorServiceInitializeTests
    Inherits BaselineSensorServiceTests

    <TestMethod(), TestCategory(Initialize), TestCategory(InvalidId)>
    Sub InitializingAnUnregisteredIdGivesAnInvalidId()
        Dim client = CreateClient()
        Dim result = client.Initialize(Guid.NewGuid.ToString).WsbdResult
        Assert.AreEqual(Status.InvalidId, result.Status)
    End Sub

    <TestMethod(), TestCategory(Initialize), TestCategory(BadValue)>
    Sub InitializingAnUnparseableSessionIdGivesABadValue()
        Dim client = CreateClient()
        client.Register()
        Dim result = client.Initialize("this_is_not_a_guid").WsbdResult
        Assert.AreEqual(Status.BadValue, result.Status)
        Assert.IsTrue(result.BadFields.Contains(Constants.SessionIdParameterName))
    End Sub

    <TestMethod(), TestCategory(Initialize), TestCategory(LockNotHeld)>
    Sub InitializationRequiresALock()
        Dim client = CreateClient()
        Dim session = client.Register.WsbdResult.SessionId.Value.ToString
        Dim result = client.Initialize(session).WsbdResult
        Assert.AreEqual(Status.LockNotHeld, result.Status)
    End Sub

    <TestMethod(), TestCategory(Initialize), TestCategory(Success)>
    Sub InitializationCanBeSuccessfull()
        Dim client = CreateClient()
        Dim session = client.Register.WsbdResult.SessionId.Value.ToString
        client.Lock(session)
        Dim result = client.Initialize(session).WsbdResult
        Assert.AreEqual(Status.Success, result.Status)
    End Sub

    <TestMethod(), TestCategory(Initialize), TestCategory(SensorFailure)>
    Sub InitializationCanResultInSensorFailure()
        Dim client = CreateClient()
        Dim session = client.Register.WsbdResult.SessionId.Value.ToString
        client.Lock(session)
        BaselineSensorService.ForceSensorFailure = True
        Dim result = client.Initialize(session).WsbdResult
        Assert.AreEqual(Status.SensorFailure, result.Status)
    End Sub

    <TestMethod(), TestCategory(Initialize), TestCategory(SensorTimeout)>
    Sub InitializationCanResultInSensorTimeout()
        Dim client = CreateClient()
        Dim session = client.Register.WsbdResult.SessionId.Value.ToString
        client.Lock(session)
        BaselineSensorService.ForceSensorTimeout = True
        Dim result = client.Initialize(session).WsbdResult
        Assert.AreEqual(Status.SensorTimeout, result.Status)
    End Sub

    <TestMethod(), TestCategory(Initialize), TestCategory(Reentrant),
    TestCategory(Success), TestCategory(SensorBusy)>
    Sub WithSimultaneousCallsOnlyOneCallInitializesSuccessfully()

        Dim busyClients As Integer = 128

        '' ----------
        '' Begin test
        '' ----------

        BaselineSensorService.InitializationTime = 10000 'ms. 

        ' Make the timeout so long that the server never Cancels the operation
        ServiceInformation.GetInstance().InitializationTimeout = BaselineSensorService.InitializationTime * 1000

        Dim mainClient = CreateClient()
        Dim mainResult As Result


        Dim sessionId = mainClient.Register.WsbdResult.SessionId.Value.ToString
        mainClient.Lock(sessionId)
        Dim runMainClient As New Thread(Sub()
                                            mainResult = mainClient.Initialize(sessionId).WsbdResult
                                        End Sub)

        ' Start a sensor operation and wait for the thread to really start
        runMainClient.Start()
        Threading.Thread.Sleep(500)


        For i As Integer = 0 To busyClients - 1
            Dim client = CreateClient()
            Dim result = client.Initialize(sessionId).WsbdResult
            Assert.AreEqual(Status.SensorBusy, result.Status)
        Next

        runMainClient.Join()

    End Sub

    <TestMethod(), TestCategory(Initialize), TestCategory(Cancel)>
    Sub InitializationCanBeCanceledSuccessfuly()


        Dim timeBeforeCancelation As Integer = 1000

        Dim client = CreateClient()
        Dim initResult As Result = Nothing
        Dim cancelResult As Result = Nothing


        Dim sessionId = client.Register.WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)

        Dim init As New Thread(Sub()
                                   initResult = client.Initialize(sessionId).WsbdResult
                               End Sub)

        Dim cancel As New Thread(Sub()
                                     Thread.Sleep(timeBeforeCancelation)
                                     cancelResult = client.Cancel(sessionId).WsbdResult

                                 End Sub)
        init.Start()
        cancel.Start()

        init.Join()
        cancel.Join()

        Assert.AreEqual(Status.Canceled, initResult.Status)
        Assert.AreEqual(Status.Success, cancelResult.Status)

    End Sub

    <TestMethod(), TestCategory(Initialize), TestCategory(Cancel)>
    Sub InitializationCanBeCanceledIdempotently()

        Dim jobCount As Integer = 100

        BaselineSensorService.InitializationTime = 2 * 1000
        Dim timeBeforeCancelation As Integer = 25

        For i As Integer = 0 To jobCount - 1

            BaselineSensorService.Reset()

            ' Performing this test a variety of times should produce a unique blend of thread interleaving

            Dim initClient = CreateClient()
            Dim cancelClient1 = CreateClient()
            Dim cancelClient2 = CreateClient()
            Dim initResult As Result = Nothing

            Dim cancelResult1 As Result = Nothing
            Dim cancelResult2 As Result = Nothing

            Dim sessionId = initClient.Register.WsbdResult.SessionId.Value.ToString
            initClient.Lock(sessionId)

            Dim init As New Thread(Sub()
                                       initResult = initClient.Initialize(sessionId).WsbdResult
                                   End Sub)

            Dim cancel1 As New Thread(Sub()
                                          Thread.Sleep(timeBeforeCancelation)
                                          cancelResult1 = cancelClient1.Cancel(sessionId).WsbdResult
                                      End Sub)

            Dim cancel2 As New Thread(Sub()
                                          Thread.Sleep(timeBeforeCancelation)
                                          cancelResult2 = cancelClient2.Cancel(sessionId).WsbdResult
                                      End Sub)
            init.Start()
            cancel1.Start()
            cancel2.Start()

            init.Join()
            cancel1.Join()
            cancel2.Join()

            initClient.Unlock(sessionId)

            Assert.AreEqual(Status.Canceled, initResult.Status)
            Assert.AreEqual(Status.Success, cancelResult1.Status)
            Assert.AreEqual(Status.Success, cancelResult2.Status)

        Next

    End Sub

    <TestMethod(), TestCategory(Initialize), TestCategory(Cancel), TestCategory(CanceledWithSensorFailure)>
    Sub CancelingInitializationCanCauseSensorFailure()

        Dim timeBeforeCancelation As Integer = 1000

        Dim client = CreateClient()
        Dim initResult As Result = Nothing
        Dim cancelResult As Result = Nothing


        ' For testing, we simulate a sensor failure by setting this 'secret' property, 
        ' and set the initialization time for much longer than we will wait to cancel it 
        ' (otherwise, the initializaiton would complete and there would be nothing to 
        ' cancel.'
        '
        BaselineSensorService.ForceSensorFailure = True
        BaselineSensorService.InitializationTime = 10 * timeBeforeCancelation

        Dim sessionId = client.Register.WsbdResult.SessionId.Value.ToString
        client.Lock(sessionId)

        Dim init As New Thread(Sub()
                                   initResult = client.Initialize(sessionId).WsbdResult
                               End Sub)

        Dim cancel As New Thread(Sub()
                                     Thread.Sleep(timeBeforeCancelation)
                                     cancelResult = client.Cancel(sessionId).WsbdResult
                                 End Sub)
        init.Start()
        cancel.Start()

        init.Join()
        cancel.Join()

        Assert.AreEqual(Status.CanceledWithSensorFailure, initResult.Status)
        Assert.AreEqual(Status.Success, cancelResult.Status)

    End Sub

    <TestMethod(), TestCategory(Initialize), TestCategory(LockHeldByAnother)>
    Sub CannotInitializeWhenLockHeldByAnother()
        Dim client1 = CreateClient()
        Dim client2 = CreateClient()
        Dim session1 = client1.Register().WsbdResult.SessionId.Value.ToString
        Dim session2 = client2.Register().WsbdResult.SessionId.Value.ToString
        client1.Lock(session1)
        Dim result = client2.Initialize(session2).WsbdResult
        Assert.AreEqual(Status.LockHeldByAnother, result.Status)
    End Sub


    <TestMethod(), TestCategory(Initialize), TestCategory(Cancel)>
    Sub ServersCanCancelInitialization()


        BaselineSensorService.InitializationTime = 1000 'ms
        ServiceInformation.GetInstance().InitializationTimeout = 50 'ms
        Dim client = CreateClient()
        Dim session = client.Register().WsbdResult.SessionId.Value.ToString
        client.Lock(session)
        Dim result = client.Initialize(session).WsbdResult

        Assert.AreEqual(Status.Canceled, result.Status)

    End Sub


End Class
